/*****************************************************************************
 *  Rohde&Schwarz HMC 8015 Power Analyzer Driver
 *  LabWindows/CVI Instrument Driver
 *  Original Release: 2015-07-10
 *  By: Stanislav Vajs
 *
 *  Modification History:
 *
 *       See ChangeLog
 *
 *****************************************************************************/

#include "rshmc8015.h"
#if (defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__))
#define ANSI
#endif
#ifndef ANSI                /* UNIX compatible */
#include <sys/timeb.h>
#include <sys/types.h>
#include <unistd.h>
#define Sleep sleep
#else                       /* ANSI compatible */
#include <windows.h>
#endif


/*****************************************************************************
 *- Value Definition and Range Tables ---------------------------------------*
 *****************************************************************************/
static ViString deviceArr[] = {"INT","EXT","DEF",NULL};
static ViString regOperArr[] = { "", "QUES", "OPER" };
static ViString regTypeArr[] = { "COND", "EVEN", "ENAB", "PTR", "NTR" };


/*****************************************************************************
 *- Callback Declarations ---------------------------------------------------*
 *****************************************************************************/

/*****************************************************************************
 *- User-Callable Functions (Exportable Functions) --------------------------*
 *****************************************************************************/

 /*****************************************************************************
  * Function: rshmc8015_init
  * Purpose:  VXIplug&play required function. Calls the
  *           rshmc8015_InitWithOptions function.
  *****************************************************************************/
ViStatus _VI_FUNC rshmc8015_init(ViRsrc resourceName, ViBoolean IDQuery,
    ViBoolean resetDevice, ViSession* newInstrSession)
{
    ViStatus error = VI_SUCCESS;

    if (newInstrSession == NULL)
    {
        (void)RsCore_SetErrorInfo(0, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
            VI_ERROR_PARAMETER4, "Null address for Instrument Handle");

        checkErr(RS_ERROR_INVALID_PARAMETER);
    }

    checkErr(rshmc8015_InitWithOptions(resourceName, IDQuery, resetDevice, "", newInstrSession));

Error:
    return error;
}

/*****************************************************************************
 * Function: rshmc8015_InitWithOptions
 * Purpose:  This function creates a new RS session and calls the
 *           RsInit function.
 *****************************************************************************/
ViStatus _VI_FUNC rshmc8015_InitWithOptions(
    ViRsrc resourceName,
    ViBoolean IDQuery,
    ViBoolean resetDevice,
    ViString optionString,
    ViSession* newInstrSession
)
{
    ViStatus error = VI_SUCCESS;
    ViSession instrSession = 0;
    RsCoreSessionPtr rsSession = NULL;
    ViChar completeOptionString[RS_MAX_MESSAGE_BUF_SIZE];
    ViPChar instrModel = NULL;

    if (newInstrSession == NULL)
    {
        (void)RsCore_SetErrorInfo(0, VI_FALSE, RS_ERROR_INVALID_PARAMETER, VI_ERROR_PARAMETER5,
            "Null address for Instrument Handle");
        checkErr(RS_ERROR_INVALID_PARAMETER);
    }

    *newInstrSession = 0;

    // TCPIP, USB and GPIB sessions to non-vxi capable
    (void)RsCore_CopyToUserBufferAsciiData(completeOptionString, RS_MAX_MESSAGE_BUF_SIZE, optionString);
    (void)RsCore_AppendToCsvString(completeOptionString, ",", RS_MAX_MESSAGE_BUF_SIZE, "AdditionalFlags=3");

    checkErr(RsCore_NewSpecificDriver(
        resourceName, // VISA Resource Name
        "rshmc8015", // driver prefix
        completeOptionString, // Init Options string, is applied to the session settings
        rshmc8015_RsCoreAttributeList, // List of all attributes from the rsxxx_AttrPropertiesList
        rshmc8015_RsCoreAttributeListLength, // Length of the attributes list
        VI_FALSE, //idnModelFullName Model has the full *IDN? query name e.g. true: "HMC8043", VI_FALSE: "HMC" Used in CheckInstrument()
        0, // WriteDelay
        0, // ReadDelay
        1000000, // IO Segment Size
        RS_VAL_OPCWAIT_STBPOLLING, // OPC Wait Mode
        RSHMC8015_OPC_TIMEOUT, // OPC timeout
        10000, // VISA Timeout
        100000, // Self-test timeout
        RS_VAL_BIN_FLOAT_FORMAT_SINGLE_4BYTES, // BinaryFloatNumbersFormat
        RS_VAL_BIN_INTEGER_FORMAT_INT32_4BYTES, // binaryIntegerNumbersFormat
        &instrSession));

    // Minimal allowed rscore version is 4.7.0
    checkErr(RsCore_CheckMinimalEngineVersion(instrSession, 4, 7, 0));

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));

    // --- Here perform settings that are default for this driver, but can be overwritten by the optionsString settings
    rsSession->assureResponseEndWithLF = VI_TRUE;
    rsSession->opcSyncQueriesMechanism = RS_OPCSYNCQUERIES_ONLYCHECKMAVERRQUEUE;

    // No SCPI command has been sent yet.
    // Call viClear before sending any SCPI command
    checkErr(RsCore_ViClear(instrSession));

    // Parse option string and set the desired values
    checkErr(RsCore_ApplyOptionString(instrSession, optionString));
    checkErr(RsCore_BuildRepCapTable(instrSession, rshmc8015_RsCoreRepCapTable, rshmc8015_RsCoreRepCapTableCount));

    // Query *IDN? string, parse it and set the following attributes:
    // - RS_ATTR_INSTRUMENT_MANUFACTURER
    // - RS_ATTR_INSTRUMENT_MODEL - based on the idnModelFullName true: "HMC8015", false : "HMC"
    // - RS_ATTR_INSTRUMENT_FIRMWARE_REVISION
    checkErr(RsCore_QueryAndParseIDNstring(instrSession, RSHMC8015_SIMULATION_ID_QUERY, NULL));

    // Not all instruments support *OPT? - only the instrument starting with NG...
    instrModel = (ViPChar)rsSession->attrInstrModelPtr->p2value;

    // Default Instrument Setup + optional *RST
    if (resetDevice == VI_TRUE)
    {
        checkErr(rshmc8015_reset(instrSession));
    }
    else
    {
        checkErr(rshmc8015_DefaultInstrSetup(instrSession));
    }

    if (IDQuery == VI_TRUE)
    {
        checkErr(RsCore_FitsIDNpattern(instrSession, RSHMC8015_VALID_ID_RESPONSE_STRING, NULL));
    }

    checkErr(rshmc8015_CheckStatus(instrSession));

Error:
    (void)RsCore_AfterErrorInInit(instrSession, rsSession, error, newInstrSession);

    return error;
}

/// HIFN  Sets acq mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR acquisitionMode/Sets acq mode.
ViStatus _VI_FUNC rshmc8015_ConfigureAcquisitionMode(ViSession instrSession,
                                                     ViInt32   channel,
                                                     ViInt32   acquisitionMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rshmc8015_channelRepCap(instrSession, channel, repCap, RS_REPCAP_BUF_SIZE));

    viCheckParm(rshmc8015_SetAttributeViInt32(instrSession, repCap, RSHMC8015_ATTR_ACQ_MODE, acquisitionMode),
            3, "Acquisition Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures AC acquisition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR pllSource/Sets PLL source.
/// HIPAR frequencyFilter/Sets state of acquisition frequency filter.
/// HIPAR digitalFilter/Sets state band width limit.
/// HIPAR bwl/Sets state of acquisition digital signal filter.
ViStatus _VI_FUNC rshmc8015_ConfigureACAcquisition(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViInt32   pllSource,
                                                   ViBoolean frequencyFilter,
                                                   ViBoolean digitalFilter,
                                                   ViBoolean bwl)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rshmc8015_channelRepCap(instrSession, channel, repCap, RS_REPCAP_BUF_SIZE));

    viCheckParm(rshmc8015_SetAttributeViInt32(instrSession, repCap, RSHMC8015_ATTR_ACQ_PLL_SOURCE, pllSource),
            3, "pll Source");

    viCheckParm(rshmc8015_SetAttributeViBoolean(instrSession, repCap, RSHMC8015_ATTR_ACQ_FREQUENCY_FILTER, frequencyFilter),
            4, "frequency Filter");

    viCheckParm(rshmc8015_SetAttributeViBoolean(instrSession, repCap, RSHMC8015_ATTR_ACQ_DIGITAL_FILTER, digitalFilter),
            5, "digital Filter");

    viCheckParm(rshmc8015_SetAttributeViBoolean(instrSession, repCap, RSHMC8015_ATTR_ACQ_BWL, bwl),
            6, "bwl");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures voltage acquisition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR autorange/Sets state of auto range for voltage range.
/// HIPAR range/Selects an appropriate voltage range for passed RMS value.
/// HIPAR crestFactor/Sets the crest factor for voltage.
/// HIPAR invert/Sets state of voltage polarity inversion.
ViStatus _VI_FUNC rshmc8015_ConfigureVoltageAcquisition(ViSession instrSession,
                                                        ViInt32   channel,
                                                        ViBoolean autorange,
                                                        ViReal64  range,
                                                        ViInt32   crestFactor,
                                                        ViBoolean invert)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rshmc8015_channelRepCap(instrSession, channel, repCap, RS_REPCAP_BUF_SIZE));

    viCheckParm(rshmc8015_SetAttributeViBoolean(instrSession, repCap, RSHMC8015_ATTR_ACQ_VOLTAGE_AUTORANGE, autorange),
            3, "autorange");

    if (autorange == VI_FALSE)
    {
        viCheckParm(rshmc8015_SetAttributeViReal64(instrSession, repCap, RSHMC8015_ATTR_ACQ_VOLTAGE_RANGE, range),
                4, "range");
    }

    viCheckParm(rshmc8015_SetAttributeViInt32(instrSession, repCap, RSHMC8015_ATTR_ACQ_VOLTAGE_CREST_FACTOR, crestFactor),
            5, "crestFactor");

    viCheckParm(rshmc8015_SetAttributeViBoolean(instrSession, repCap, RSHMC8015_ATTR_ACQ_VOLTAGE_INVERT, invert),
            6, "invert");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures current acquisition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR autorange/Sets state of auto range for current range.
/// HIPAR range/Selects an appropriate current range for passed RMS value.
/// HIPAR crestFactor/Sets the crest factor for current.
/// HIPAR invert/Sets state of current polarity inversion.
ViStatus _VI_FUNC rshmc8015_ConfigureCurrentAcquisition(ViSession instrSession,
                                                        ViInt32   channel,
                                                        ViBoolean autorange,
                                                        ViReal64  range,
                                                        ViInt32   crestFactor,
                                                        ViBoolean invert)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rshmc8015_channelRepCap(instrSession, channel, repCap, RS_REPCAP_BUF_SIZE));

    viCheckParm(rshmc8015_SetAttributeViBoolean(instrSession, repCap, RSHMC8015_ATTR_ACQ_CURRENT_AUTORANGE, autorange),
            3, "autorange");

    if (autorange == VI_FALSE)
    {
        viCheckParm(rshmc8015_SetAttributeViReal64(instrSession, repCap, RSHMC8015_ATTR_ACQ_CURRENT_RANGE, range),
                4, "range");
    }

    viCheckParm(rshmc8015_SetAttributeViInt32(instrSession, repCap, RSHMC8015_ATTR_ACQ_CURRENT_CREST_FACTOR, crestFactor),
            5, "crestFactor");

    viCheckParm(rshmc8015_SetAttributeViBoolean(instrSession, repCap, RSHMC8015_ATTR_ACQ_CURRENT_INVERT, invert),
            6, "invert");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Query whether overcurrent protection was triggered.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR ocp/Query whether overcurrent protection was triggered.
ViStatus _VI_FUNC rshmc8015_QueryOCP(ViSession  instrSession,
                                     ViInt32    channel,
                                     ViBoolean* ocp)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rshmc8015_channelRepCap(instrSession, channel, repCap, RS_REPCAP_BUF_SIZE));

    viCheckParm(rshmc8015_GetAttributeViBoolean(instrSession, repCap, RSHMC8015_ATTR_ACQ_OCP, ocp),
            3, "OCP");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Reset overcurrent protection.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
ViStatus _VI_FUNC rshmc8015_ResetOCP(ViSession instrSession,
                                     ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rshmc8015_channelRepCap(instrSession, channel, repCap, RS_REPCAP_BUF_SIZE));

    checkErr(rshmc8015_SetAttributeViString(instrSession, repCap, RSHMC8015_ATTR_ACQ_OCP_RESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays a text message box on the front display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR text/The text to display in the message box.
ViStatus _VI_FUNC rshmc8015_ShowMessageBox(ViSession instrSession,
                                           ViString  text)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc8015_SetAttributeViString(instrSession, "", RSHMC8015_ATTR_DISPLAY_TEXT, text),
            2, "Text");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Clears the text message box on the front display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rshmc8015_ClearMessageBox(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rshmc8015_SetAttributeViString(instrSession, "", RSHMC8015_ATTR_DISPLAY_CLEAR, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets view mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR viewMode/Sets view mode.
ViStatus _VI_FUNC rshmc8015_ConfigureViewMode(ViSession instrSession,
                                              ViInt32   viewMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc8015_SetAttributeViInt32(instrSession, "", RSHMC8015_ATTR_VIEW_MODE, viewMode),
            2, "View Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets page's cell count.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR page/Numerical selection of a page
/// HIPAR size/Sets page's cell count.
ViStatus _VI_FUNC rshmc8015_ConfigureNormalViewPageSize(ViSession instrSession,
                                                        ViInt32   page,
                                                        ViInt32   size)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, page, 1, 4),
            2, "Page");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Page%ld", page);

    viCheckParm(rshmc8015_SetAttributeViInt32(instrSession, repCap, RSHMC8015_ATTR_VIEW_NORM_PAGE_SIZE, size),
            3, "Size");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects active page.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR activePage/Selects active page.
ViStatus _VI_FUNC rshmc8015_ConfigureNormalViewActivePage(ViSession instrSession,
                                                          ViInt32   activePage)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc8015_SetAttributeViInt32(instrSession, "", RSHMC8015_ATTR_VIEW_NORM_ACTIVE_PAGE, activePage),
            2, "Cctive Page");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the normal view cell function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR page/Numerical selection of a page
/// HIPAR cell/Numerical selection of a cell
/// HIPAR function/Selects the normal view cell function.
ViStatus _VI_FUNC rshmc8015_ConfigureNormalViewFunction(ViSession instrSession,
                                                        ViInt32   page,
                                                        ViInt32   cell,
                                                        ViInt32   function)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, page, 1, 4),
            2, "Page");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, cell, 1, 6),
            3, "Cell");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Page%ld,Cell%ld", page, cell);
    viCheckParm(rshmc8015_SetAttributeViInt32(instrSession, repCap, RSHMC8015_ATTR_VIEW_NORM_FUNCTION, function),
            3, "Function");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures harmonics view.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR select/Selects harmonics view.
/// HIPAR source/Selects harmonics source.
/// HIPAR number/Sets number of harmonics.
/// HIPAR scaling/Selects harmonics scaling.
/// HIPAR subset/Selects harmonics subset.
ViStatus _VI_FUNC rshmc8015_ConfigureHarmonicsView(ViSession instrSession,
                                                   ViInt32   select,
                                                   ViInt32   source,
                                                   ViInt32   number,
                                                   ViInt32   scaling,
                                                   ViInt32   subset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc8015_SetAttributeViInt32(instrSession, "", RSHMC8015_ATTR_VIEW_HARM_SELECT, select),
            2, "select");

    viCheckParm(rshmc8015_SetAttributeViInt32(instrSession, "", RSHMC8015_ATTR_VIEW_HARM_SOURCE, source),
            3, "source");

    viCheckParm(rshmc8015_SetAttributeViInt32(instrSession, "", RSHMC8015_ATTR_VIEW_HARM_NUMBER, number),
            4, "number");

    viCheckParm(rshmc8015_SetAttributeViInt32(instrSession, "", RSHMC8015_ATTR_VIEW_HARM_SCALING, scaling),
            5, "scaling");

    viCheckParm(rshmc8015_SetAttributeViInt32(instrSession, "", RSHMC8015_ATTR_VIEW_HARM_SUBSET, subset),
            6, "subset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries measurements specified via ConfigureFunctionList method.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR arrayLength/Number of array points requested.
/// HIPAR data/Queries measurements specified via ConfigureFunctionList method.
/// HIPAR actualDataLength/Number of points actually returned in the array.
ViStatus _VI_FUNC rshmc8015_QueryData(ViSession instrSession,
                                      ViInt32   channel,
                                      ViInt32   arrayLength,
                                      ViReal64  data[],
                                      ViInt32*  actualDataLength)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 4),
            2, "channel");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CHAN%ld:MEAS:FORM ASC", channel);
    checkErr(RsCore_Write(instrSession, cmd));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CHAN%ld:MEAS:DATA?", channel);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arrayLength, data, actualDataLength));

    checkErr(rshmc8015_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets list of function names.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR functionList/List of function names.
ViStatus _VI_FUNC rshmc8015_WriteFunctionList(ViSession instrSession,
                                              ViInt32   channel,
                                              ViString  functionList)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 4),
            2, "channel");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CHAN%ld:MEAS:FUNC %s", channel, functionList);
    checkErr(rshmc8015_WriteInstrData(instrSession, cmd));

    checkErr(rshmc8015_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries list of function name.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Output parameter.
/// HIPAR functionList/This indicator returns the function names as comma separated strings,
/// HIPAR functionList/for example: URMS,P,TEMP1
ViStatus _VI_FUNC rshmc8015_QueryFunctionList(ViSession instrSession,
                                              ViInt32   channel,
                                              ViInt32   bufferSize,
                                              ViChar    functionList[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 4),
            2, "channel");
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CHAN%ld:MEAS:FUNC?", channel);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, functionList, NULL));

    checkErr(rshmc8015_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function returns the length of function list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR length/Returns the length of function list.
ViStatus _VI_FUNC rshmc8015_QueryFunctionListLength(ViSession instrSession,
                                                    ViInt32   channel,
                                                    ViInt32*  length)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rshmc8015_channelRepCap(instrSession, channel, repCap, RS_REPCAP_BUF_SIZE));

    viCheckParm(rshmc8015_GetAttributeViInt32(instrSession, repCap, RSHMC8015_ATTR_MEAS_FUNCTION_LIST_LENGTH, length),
            3, "length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the global data integrator function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Sets Integrator state (enables or disables the Integrator).
/// HIPAR mode/Sets the integration mode.
/// HIPAR duration/Sets the integration timer value.
ViStatus _VI_FUNC rshmc8015_ConfigureIntegrator(ViSession instrSession,
                                                ViBoolean enabled,
                                                ViInt32   mode,
                                                ViReal64  duration)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc8015_SetAttributeViBoolean(instrSession, "", RSHMC8015_ATTR_INTEGRATOR_STATE, enabled),
            2, "enabled");

    viCheckParm(rshmc8015_SetAttributeViInt32(instrSession, "", RSHMC8015_ATTR_INTEGRATOR_MODE, mode),
            3, "mode");

    viCheckParm(rshmc8015_SetAttributeViReal64(instrSession, "", RSHMC8015_ATTR_INTEGRATOR_DURATION, duration),
            4, "duration");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function specifies the type of integrator operation and performs
/// HIFN  the operation.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSHMC8015_ATTR_INTEGRATOR_START
/// HIFN     RSHMC8015_ATTR_INTEGRATOR_STOP
/// HIFN     RSHMC8015_ATTR_INTEGRATOR_RESET
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     INTegrator:STARt
/// HIFN     INTegrator:STOP
/// HIFN     INTegrator:RESet
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR operation/Selects the integration operation.
ViStatus _VI_FUNC rshmc8015_IntegratorOperation(ViSession instrSession,
                                                ViInt32   operation)
{
ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (operation)
    {
        case RSHMC8015_VAL_INTEGRATOR_OPERATION_START:
            checkErr(rshmc8015_SetAttributeViString(instrSession, "", RSHMC8015_ATTR_INTEGRATOR_START, NULL));
            break;
        case RSHMC8015_VAL_INTEGRATOR_OPERATION_STOP:
            checkErr(rshmc8015_SetAttributeViString(instrSession, "", RSHMC8015_ATTR_INTEGRATOR_STOP, NULL));
            break;
        case RSHMC8015_VAL_INTEGRATOR_OPERATION_RESET:
            checkErr(rshmc8015_SetAttributeViString(instrSession, "", RSHMC8015_ATTR_INTEGRATOR_RESET, NULL));
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, operation), 2, "Operation");
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets integrator's start time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR year/This parameter representing the year.
/// HIPAR month/This parameter representing the month of the year.
/// HIPAR day/This parameter representing the day of the month.
/// HIPAR hour/This parameter representing the hour.
/// HIPAR minute/This parameter representing the minutes.
/// HIPAR second/This parameter representing the seconds.
ViStatus _VI_FUNC rshmc8015_ConfigureIntegratorStartTime(ViSession instrSession,
                                                         ViInt32   year,
                                                         ViInt32   month,
                                                         ViInt32   day,
                                                         ViInt32   hour,
                                                         ViInt32   minute,
                                                         ViInt32   second)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "INT:STIME %ld,%ld,%ld,%ld,%ld,%ld",
            year, month, day, hour, minute, second);
    checkErr(rshmc8015_WriteInstrData(instrSession, cmd));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Set the filename and device for logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filename/Selects the filename.
/// HIPAR device/Selects the device.
ViStatus _VI_FUNC rshmc8015_ConfigureLoggingFile(ViSession instrSession,
                                                 ViString  filename,
                                                 ViInt32   device)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LOG:FNAM '%s',%s", filename, deviceArr[device]);
    checkErr(rshmc8015_WriteInstrData(instrSession, cmd));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the global data logging function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Sets logging state (starts or stops a logging session).
/// HIPAR mode/Selects the mode of the logging function.
/// HIPAR count/Sets maximum number of data rows of a log file.
/// HIPAR time/Sets the time in "TIME" mode.
ViStatus _VI_FUNC rshmc8015_ConfigureLogging(ViSession instrSession,
                                             ViBoolean enabled,
                                             ViInt32   mode,
                                             ViInt32   count,
                                             ViReal64  time)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc8015_SetAttributeViBoolean(instrSession, "", RSHMC8015_ATTR_LOGGING_STATE, enabled),
            2, "enabled");

    viCheckParm(rshmc8015_SetAttributeViInt32(instrSession, "", RSHMC8015_ATTR_LOGGING_MODE, mode),
            3, "mode");

    viCheckParm(rshmc8015_SetAttributeViInt32(instrSession, "", RSHMC8015_ATTR_LOGGING_COUNT, count),
            4, "count");

    viCheckParm(rshmc8015_SetAttributeViReal64(instrSession, "", RSHMC8015_ATTR_LOGGING_TIME, time),
            5, "time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets logging start time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR year/This parameter representing the year.
/// HIPAR month/This parameter representing the month of the year.
/// HIPAR day/This parameter representing the day of the month.
/// HIPAR hour/This parameter representing the hour.
/// HIPAR minute/This parameter representing the minutes.
/// HIPAR second/This parameter representing the seconds.
ViStatus _VI_FUNC rshmc8015_ConfigureLoggingStartTime(ViSession instrSession,
                                                      ViInt32   year,
                                                      ViInt32   month,
                                                      ViInt32   day,
                                                      ViInt32   hour,
                                                      ViInt32   minute,
                                                      ViInt32   second)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LOG:STIME %ld,%ld,%ld,%ld,%ld,%ld",
            year, month, day, hour, minute, second);
        checkErr(rshmc8015_WriteInstrData(instrSession, cmd));

    checkErr(rshmc8015_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the result of limit check.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR limitCheckResult/Queries the result of limit check.
ViStatus _VI_FUNC rshmc8015_QueryLimitCheckResult(ViSession  instrSession,
                                                  ViInt32    channel,
                                                  ViInt32    limit,
                                                  ViBoolean* limitCheckResult)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 1),
            2, "channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, limit, 1, 6),
            3, "limit");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "CH%ld,Lim%ld", channel, limit);
    viCheckParm(rshmc8015_GetAttributeViBoolean(instrSession, repCap, RSHMC8015_ATTR_LIMIT_CHECK_RESULT, limitCheckResult),
            4, "Limit Check Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the result of limit calculation between min and max values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR limitPercent/Queries the result of limit calculation between min and max values.
ViStatus _VI_FUNC rshmc8015_QueryLimitPercent(ViSession instrSession,
                                              ViInt32   channel,
                                              ViInt32   limit,
                                              ViReal64* limitPercent)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 1),
            2, "channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, limit, 1, 6),
            3, "limit");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "CH%ld,Lim%ld", channel, limit);

    viCheckParm(rshmc8015_GetAttributeViReal64(instrSession, repCap, RSHMC8015_ATTR_LIMIT_PERCENT, limitPercent),
            4, "Limit Percent");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures measurement limit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR limit/Numerical selection of a limit
/// HIPAR state/Sets state of the limit.
/// HIPAR source/Sets source (function) of the limit.
/// HIPAR high/Sets limit's upper boundary.
/// HIPAR low/Sets limit's lower boundary.
ViStatus _VI_FUNC rshmc8015_ConfigureMeasurementLimit(ViSession instrSession,
                                                      ViInt32   channel,
                                                      ViInt32   limit,
                                                      ViBoolean state,
                                                      ViInt32   source,
                                                      ViReal64  high,
                                                      ViReal64  low)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 1),
            2, "channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, limit, 1, 6),
            3, "limit");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "CH%ld,Lim%ld", channel, limit);

    viCheckParm(rshmc8015_SetAttributeViBoolean(instrSession, repCap, RSHMC8015_ATTR_LIMIT_STATE, state),
            4, "state");

    viCheckParm(rshmc8015_SetAttributeViInt32(instrSession, repCap, RSHMC8015_ATTR_LIMIT_SOURCE, source),
            5, "source");

    viCheckParm(rshmc8015_SetAttributeViReal64(instrSession, repCap, RSHMC8015_ATTR_LIMIT_HIGH, high),
            6, "high");

    viCheckParm(rshmc8015_SetAttributeViReal64(instrSession, repCap, RSHMC8015_ATTR_LIMIT_LOW, low),
            7, "low");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures analog input settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR state/Sets enabled state of the analog input.
ViStatus _VI_FUNC rshmc8015_ConfigureExtAnalogIn(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rshmc8015_channelRepCap(instrSession, channel, repCap, RS_REPCAP_BUF_SIZE));

    viCheckParm(rshmc8015_SetAttributeViBoolean(instrSession, repCap, RSHMC8015_ATTR_EXT_AINPUT_STATE, state),
            3, "state");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures analog output settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR state/Sets enabled state of the analog output.
/// HIPAR mode/Sets analog output mode.
ViStatus _VI_FUNC rshmc8015_ConfigureExtAnalogOut(ViSession instrSession,
                                                  ViInt32   channel,
                                                  ViBoolean state,
                                                  ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rshmc8015_channelRepCap(instrSession, channel, repCap, RS_REPCAP_BUF_SIZE));

    viCheckParm(rshmc8015_SetAttributeViBoolean(instrSession, repCap, RSHMC8015_ATTR_EXT_AOUTPUT_STATE, state),
            3, "state");

    viCheckParm(rshmc8015_SetAttributeViInt32(instrSession, repCap, RSHMC8015_ATTR_EXT_AOUTPUT_MODE, mode),
            4, "mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures digital input settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR state/Sets enabled state of the digital input.
/// HIPAR mode/Sets digital input mode.
/// HIPAR invert/Sets negation of ext. digital input.
ViStatus _VI_FUNC rshmc8015_ConfigureExtDigitalIn(ViSession instrSession,
                                                  ViInt32   channel,
                                                  ViBoolean state,
                                                  ViInt32   mode,
                                                  ViBoolean invert)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rshmc8015_channelRepCap(instrSession, channel, repCap, RS_REPCAP_BUF_SIZE));

    viCheckParm(rshmc8015_SetAttributeViBoolean(instrSession, repCap, RSHMC8015_ATTR_EXT_DINPUT_STATE, state),
            3, "state");

    viCheckParm(rshmc8015_SetAttributeViInt32(instrSession, repCap, RSHMC8015_ATTR_EXT_DINPUT_MODE, mode),
            4, "mode");

    viCheckParm(rshmc8015_SetAttributeViBoolean(instrSession, repCap, RSHMC8015_ATTR_EXT_DINPUT_INVERT, invert),
            5, "invert");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures digital output settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR state/Sets enabled state of the digital output.
/// HIPAR mode/Sets digital output mode.
/// HIPAR invert/Sets negation of ext. digital output.
/// HIPAR outputState/Sets digital output state.
ViStatus _VI_FUNC rshmc8015_ConfigureExtDigitalOut(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViBoolean state,
                                                   ViInt32   mode,
                                                   ViBoolean invert,
                                                   ViBoolean outputState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rshmc8015_channelRepCap(instrSession, channel, repCap, RS_REPCAP_BUF_SIZE));

    viCheckParm(rshmc8015_SetAttributeViBoolean(instrSession, repCap, RSHMC8015_ATTR_EXT_DOUTPUT_STATE, state),
            3, "state");

    viCheckParm(rshmc8015_SetAttributeViInt32(instrSession, repCap, RSHMC8015_ATTR_EXT_DOUTPUT_MODE, mode),
            4, "mode");

    viCheckParm(rshmc8015_SetAttributeViBoolean(instrSession, repCap, RSHMC8015_ATTR_EXT_DOUTPUT_INVERT, invert),
            5, "invert");

    viCheckParm(rshmc8015_SetAttributeViBoolean(instrSession, repCap, RSHMC8015_ATTR_EXT_DOUTPUT_OUTPUT_STATE, outputState),
            6, "Output State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures external sensor settings.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSHMC8015_ATTR_EXT_SENSOR_STATE
/// HIFN     RSHMC8015_ATTR_EXT_SENSOR_MODE
/// HIFN     RSHMC8015_ATTR_EXT_SENSOR_SHUNT_RESISTANCE
/// HIFN     RSHMC8015_ATTR_EXT_SENSOR_CLAMP_RATIO
/// HIFN     RSHMC8015_ATTR_EXT_SENSOR_RANGE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CHANnel<n>[:EXTern]:SENSor[:STATe]
/// HIFN     CHANnel<n>[:EXTern]:SENSor:MODE
/// HIFN     CHANnel<n>[:EXTern]:SENSor:EXTShunt[:RESistance]
/// HIFN     CHANnel<n>[:EXTern]:SENSor:CCLamp[:RATio]
/// HIFN     CHANnel<n>[:EXTern]:SENSor:RANGe
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Numerical selection of a channel
/// HIPAR state/Sets enabled state of the sensor.
/// HIPAR mode/Sets sensor mode.
/// HIPAR resistance/Sets resistance of the external shunt.
/// HIPAR ratio/Sets V/A ratio of the current clamp.
/// HIPAR range/Selects an appropriate sensor voltage range for passed peak voltage
/// HIPAR range/value.
ViStatus _VI_FUNC rshmc8015_ConfigureExtSensor(ViSession instrSession,
                                               ViInt32   channel,
                                               ViBoolean state,
                                               ViInt32   mode,
                                               ViReal64  resistance,
                                               ViReal64  ratio,
                                               ViReal64  range)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rshmc8015_channelRepCap(instrSession, channel, repCap, RS_REPCAP_BUF_SIZE));

    viCheckParm(rshmc8015_SetAttributeViBoolean(instrSession, repCap, RSHMC8015_ATTR_EXT_SENSOR_STATE, state),
            3, "state");

    viCheckParm(rshmc8015_SetAttributeViInt32(instrSession, repCap, RSHMC8015_ATTR_EXT_SENSOR_MODE, mode),
            4, "mode");

    viCheckParm(rshmc8015_SetAttributeViReal64(instrSession, repCap, RSHMC8015_ATTR_EXT_SENSOR_SHUNT_RESISTANCE, resistance),
            5, "resistance");

    viCheckParm(rshmc8015_SetAttributeViReal64(instrSession, repCap, RSHMC8015_ATTR_EXT_SENSOR_CLAMP_RATIO, ratio),
            6, "ratio");

    viCheckParm(rshmc8015_SetAttributeViReal64(instrSession, repCap, RSHMC8015_ATTR_EXT_SENSOR_RANGE, range),
            7, "range");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the value of a ViInt32 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the
/// HIPAR channelName/attribute is not channel-based, then pass NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rshmc8015_SetAttributeViInt32(ViSession instrSession,
                                                ViString  repeatedCapabilityName,
                                                ViAttr    attributeId,
                                                ViInt32   attributeValue)
{
    return RsCore_SetAttributeViInt32(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);
}

/// HIFN  This function sets the value of a ViInt64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the
/// HIPAR channelName/attribute is not channel-based, then pass NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rshmc8015_SetAttributeViInt64(ViSession instrSession,
                                                ViChar    repeatedCapabilityName[],
                                                ViAttr    attributeId,
                                                ViInt64   attributeValue)
{
    return RsCore_SetAttributeViInt64(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);
}

/// HIFN  This function sets the value of a ViReal64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the
/// HIPAR channelName/attribute is not channel-based, then pass NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rshmc8015_SetAttributeViReal64(ViSession instrSession,
                                                 ViString  repeatedCapabilityName,
                                                 ViAttr    attributeId,
                                                 ViReal64  attributeValue)
{
     return RsCore_SetAttributeViReal64(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);
}

/// HIFN  This function sets the value of a ViString attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the
/// HIPAR channelName/attribute is not channel-based, then pass NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rshmc8015_SetAttributeViString(ViSession instrSession,
                                                 ViString  repeatedCapabilityName,
                                                 ViAttr    attributeId,
                                                 ViString  attributeValue)
{
    return RsCore_SetAttributeViString(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);
}

/// HIFN  This function sets the value of a ViBoolean attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the
/// HIPAR channelName/attribute is not channel-based, then pass NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rshmc8015_SetAttributeViBoolean(ViSession instrSession,
                                                  ViString  repeatedCapabilityName,
                                                  ViAttr    attributeId,
                                                  ViBoolean attributeValue)
{
    return RsCore_SetAttributeViBoolean(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);
}

/// HIFN  This function queries the value of a ViInt32 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the
/// HIPAR channelName/attribute is not channel-based, then pass NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a
/// HIPAR value/ViInt32 variable.
ViStatus _VI_FUNC rshmc8015_GetAttributeViInt32(ViSession instrSession,
                                                ViString  repeatedCapabilityName,
                                                ViAttr    attributeId,
                                                ViInt32   *value)
{
    return RsCore_GetAttributeViInt32(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/// HIFN  This function queries the value of a ViInt64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsavpavda_init or
/// HIPAR instrSession/rsavpavda_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the
/// HIPAR channelName/attribute is not channel-based, then pass NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a
/// HIPAR value/ViInt64 variable.
ViStatus _VI_FUNC rshmc8015_GetAttributeViInt64(ViSession instrSession,
                                                ViString  repeatedCapabilityName,
                                                ViAttr    attributeId,
                                                ViInt64   *value)
{
    return RsCore_GetAttributeViInt64(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL,value);
}

/// HIFN  This function queries the value of a ViReal64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the
/// HIPAR channelName/attribute is not channel-based, then pass NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a
/// HIPAR value/ViReal64 variable.
ViStatus _VI_FUNC rshmc8015_GetAttributeViReal64(ViSession instrSession,
                                                 ViString  repeatedCapabilityName,
                                                 ViAttr    attributeId,
                                                 ViReal64  *value)
{
    return RsCore_GetAttributeViReal64(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/// HIFN  This function queries the value of a ViString attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the
/// HIPAR channelName/attribute is not channel-based, then pass NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR bufSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufSize/Attribute Value parameter.
/// HIPAR value/The buffer in which the function returns the current value of the
/// HIPAR value/attribute.  The buffer must be of type ViChar and have at least as many
/// HIPAR value/bytes as indicated in the Buffer Size parameter.
ViStatus _VI_FUNC rshmc8015_GetAttributeViString(ViSession instrSession,
                                                 ViString  repeatedCapabilityName,
                                                 ViAttr    attributeId,
                                                 ViInt32   bufferSize,
                                                 ViChar    attributeValue[])
{
    return RsCore_GetAttributeViString(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, bufferSize, attributeValue);
}

/// HIFN  This function queries the value of a ViBoolean attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the
/// HIPAR channelName/attribute is not channel-based, then pass NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a
/// HIPAR value/ViBoolean variable.
ViStatus _VI_FUNC rshmc8015_GetAttributeViBoolean(ViSession instrSession,
                                                  ViString  repeatedCapabilityName,
                                                  ViAttr    attributeId,
                                                  ViBoolean *value)
{
    return RsCore_GetAttributeViBoolean(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/*****************************************************************************
 * Function: rshmc8015_GetAttributeRepeatedCapabilityIds
 * Purpose:  This function returns the repeated capability id(s) that
 *           belongs to the attribute you specify. More then one id is listed
 *           in comma separated string.
 *****************************************************************************/
ViStatus _VI_FUNC rshmc8015_GetAttributeRepeatedCapabilityIds(
                            ViSession instrSession,
                            ViUInt32  attributeID,
                            ViInt32   bufferSize,
                            ViChar    repeatedCapabilityIds[])
{
    ViStatus error = VI_SUCCESS;

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bufferSize, 1, RS_MAX_SHORT_MESSAGE_BUF_SIZE),
            3, "Buffer Size");

    checkErr(RsCore_GetAttributeRepCapNameIds(instrSession, attributeID, bufferSize, repeatedCapabilityIds));

Error:
    return error;
}

/*****************************************************************************
 * Function: Get Attribute Repeated Capability Id Name(s)
 * Purpose:  This function returns the repeated capability name(s) that
 *           belongs to the attribute and single repeated capability id you
 *           may specify. More then one name is listed in comma separated
 *           string.
 *****************************************************************************/
ViStatus _VI_FUNC rshmc8015_GetAttributeRepeatedCapabilityIdNames(
                            ViSession instrSession,
                            ViUInt32  attributeID,
                            ViString  repeatedCapabilityId,
                            ViInt32   bufferSize,
                            ViChar    repeatedCapabilityIdNames[])
{
    ViStatus error = VI_SUCCESS;

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bufferSize, 1, RS_MAX_SHORT_MESSAGE_BUF_SIZE),
            4, "Buffer Size");

    checkErr(RsCore_GetAttributeRepCapNamesAll(instrSession,
        attributeID,
        repeatedCapabilityId,
        bufferSize,
        repeatedCapabilityIdNames));

Error:
    return error;
}

/// HIFN Function used by Hi-level functions to obtain a repCap name from:
/// HIFN - attributeID
/// HIFN - repCapNameID
/// HIFN - 0-based index in the repCapNames
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR repCapNameId/RepCap Name ID.
/// HIPAR index/Zero based index in the repCapNames.
/// HIPAR bufferSize/Pass the number of bytes in the string you allocate for the repCapName.
/// HIPAR repCapName/String returning RepCap name
ViStatus _VI_FUNC rshmc8015_GetAttributeRepCapName(ViSession instrSession,
                                                   ViUInt32  attributeId,
                                                   ViString  repCapNameId,
                                                   ViInt32   index,
                                                   ViInt32   arraySize,
                                                   ViChar    repCapName[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, attributeId, repCapNameId, index, arraySize, repCapName));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Reset
 * Purpose:  This function resets the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rshmc8015_reset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (RsCore_Simulating(instrSession) == VI_FALSE)
    {
        checkErr(RsCore_ClearStatus(instrSession));
        checkErr(RsCore_Write(instrSession, "*RST")); // IGNORE: Check double write
        checkErr(RsCore_QueryViStringShort(instrSession, "*OPC?", NULL));
    }

    checkErr(rshmc8015_DefaultInstrSetup(instrSession));
    checkErr(rshmc8015_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}


/*****************************************************************************
 * Function: Self Test
 * Purpose:  This function executes the instrument self-test and returns the
 *           result.
 *****************************************************************************/
ViStatus _VI_FUNC rshmc8015_self_test(ViSession instrSession,
                                      ViInt16*  testResult,
                                      ViChar    testMessage[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, testResult), 2, "Null address for Test Result");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, testMessage), 3, "Null address for Test Message");

    checkErr(RsCore_SelfTest(instrSession, testResult, 48, testMessage));

    checkErr(rshmc8015_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Revision Query
 * Purpose:  This function returns the driver and instrument revisions.
 *****************************************************************************/
ViStatus _VI_FUNC rshmc8015_revision_query(ViSession instrSession,
                                           ViChar    instrumentDriverRevision[],
                                           ViChar    firmwareRevision[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, instrumentDriverRevision), 2, "Null address for Instrument Driver Revision");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, firmwareRevision), 3, "Null address for Instrument Revision");

    checkErr(RsCore_GetAttributeViString(instrSession, NULL, RS_ATTR_SPECIFIC_DRIVER_REVISION, 0, 256, instrumentDriverRevision));
    checkErr(RsCore_GetAttributeViString(instrSession, NULL, RS_ATTR_INSTRUMENT_FIRMWARE_REVISION, 0, 256, firmwareRevision));
    checkErr(rshmc8015_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Disables or enables the beeper tone.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR beeperState/Disables or enables the beeper tone.
ViStatus _VI_FUNC rshmc8015_BeeperState(ViSession instrSession,
                                        ViBoolean beeperState)
{
     ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc8015_SetAttributeViBoolean(instrSession, "", RSHMC8015_ATTR_SYSTEM_BEEPER_ENABLED, beeperState),
            2, "Beeper State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  When sending a command, the device emits a beep.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rshmc8015_StartBeep(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rshmc8015_SetAttributeViString(instrSession, "", RSHMC8015_ATTR_SYSTEM_START_BEEP, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the name of the device (capped at 20 characters).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR name/The name of system.
ViStatus _VI_FUNC rshmc8015_SystemName(ViSession instrSession,
                                       ViString  name)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc8015_SetAttributeViString(instrSession, "", RSHMC8015_ATTR_SYSTEM_NAME, name),
            2, "Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Local operating mode (front panel control).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rshmc8015_SystemLocal(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rshmc8015_SetAttributeViString(instrSession, "", RSHMC8015_ATTR_SYSTEM_LOCAL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Remote control (front panel control locked).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rshmc8015_SystemRemote(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rshmc8015_SetAttributeViString(instrSession, "", RSHMC8015_ATTR_SYSTEM_REMOTE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  The front panel control is locked; with the SystemLocal method the
/// HIFN  front panel control will be activated.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSHMC8015_ATTR_SYSTEM_RWLOCK
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SYSTem:RWLock
/// HIFN
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rshmc8015_SystemRWLock(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rshmc8015_SetAttributeViString(instrSession, "", RSHMC8015_ATTR_SYSTEM_RWLOCK, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the number of the SCPI version, which is
/// HIFN  relevant for the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR length/This control specifies the size of the buffer.
/// HIPAR systemVersion/Returns the number of the SCPI version, which is relevant for the
/// HIPAR systemVersion/instrument.
ViStatus _VI_FUNC rshmc8015_SystemVersion(ViSession instrSession,
                                          ViInt32   length,
                                          ViChar    systemVersion[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc8015_GetAttributeViString(instrSession, "", RSHMC8015_ATTR_SYSTEM_VERSION,
            length, systemVersion),
            3, "System Version");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the device model.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR length/Sets the allocated memory length of System Version string.
/// HIPAR systemDevice/Returns the system device model.
ViStatus _VI_FUNC rshmc8015_SystemDevice(ViSession instrSession,
                                         ViInt32   length,
                                         ViChar    systemDevice[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc8015_GetAttributeViString(instrSession, "", RSHMC8015_ATTR_SYSTEM_DEVICE,
            length, systemDevice),
            3, "System Device");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Turns the device off. The device can't be enabled remotely after this
/// HIFN  command.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rshmc8015_SystemShutdown(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rshmc8015_SetAttributeViString(instrSession, "", RSHMC8015_ATTR_SYSTEM_SHUTDOWN, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the software version.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR length/Sets the allocated memory length of System Version string.
/// HIPAR systemSoftware/Returns the system software version.
ViStatus _VI_FUNC rshmc8015_SystemSoftware(ViSession instrSession,
                                           ViInt32   length,
                                           ViChar    systemSoftware[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc8015_GetAttributeViString(instrSession, "", RSHMC8015_ATTR_SYSTEM_SOFTWARE,
            length, systemSoftware),
            3, "System Software");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the hardware version.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR length/Sets the allocated memory length of System Version string.
/// HIPAR systemHardware/Returns the system hardware version.
ViStatus _VI_FUNC rshmc8015_SystemHardware(ViSession instrSession,
                                           ViInt32   length,
                                           ViChar    systemHardware[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc8015_GetAttributeViString(instrSession, "", RSHMC8015_ATTR_SYSTEM_HARDWARE,
            length, systemHardware),
            3, "System Hardware");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the serial number.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR length/Sets the allocated memory length of System Version string.
/// HIPAR systemSerialNumber/Returns the system serial number.
ViStatus _VI_FUNC rshmc8015_SystemSerialNumber(ViSession instrSession,
                                               ViInt32   length,
                                               ViChar    systemSerialNumber[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc8015_GetAttributeViString(instrSession, "", RSHMC8015_ATTR_SYSTEM_SERIAL_NUMBER,
            length, systemSerialNumber),
            3, "System Serial Number");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function reads an error code and a message from the
/// HIFN instrument's error queue.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.
/// HIPAR errCode/Returns the error code read from the instrument's error queue.
/// HIPAR errMessage/Returns the error message string read from the instrument's error message queue.
ViStatus _VI_FUNC rshmc8015_error_query(ViSession instrSession,
                                        ViInt32   * errorCode,
                                        ViChar    errorMessage[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorCode), 2, "Null address for Error Code");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorMessage), 3, "Null address for Error Message");

    checkErr(RsCore_ErrorQueryAll(instrSession, errorCode, 1024, errorMessage));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function converts a status code returned by an instrument driver function into a user-readable string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.
/// HIPAR errorCode/Pass the Status parameter that is returned from any of the instrument driver functions.
/// HIPAR errorMessage/Returns the user-readable message string that corresponds to the status code you specify.
ViStatus _VI_FUNC rshmc8015_error_message(ViSession instrSession,
                                          ViStatus  errorCode,
                                          ViChar    errorMessage[])
{
    return rshmc8015_utility_error_message(instrSession, errorCode, errorMessage);
}

/// HIFN  This function sets the Enable of selected registers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR registerOperation/This control selects the registers that will be configured.
/// HIPAR enable/Sets content of enable register.
/// HIPAR ptransition/Sets content of PTransition register.
/// HIPAR ntransition/Sets content of NTransition register.
ViStatus _VI_FUNC rshmc8015_SetStatusRegister(ViSession instrSession,
                                              ViInt32   registerOperation,
                                              ViInt32   enable,
                                              ViInt32   ptransition,
                                              ViInt32   ntransition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, registerOperation, 1, 2),
            2, "registerOperation");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "STAT:%s:ENAB %ld", regOperArr[registerOperation], enable);
    checkErr(RsCore_Write(instrSession, cmd));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "STAT:%s:PTR %ld", regOperArr[registerOperation], ptransition);
    checkErr(RsCore_Write(instrSession, cmd));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "STAT:%s:NTR %ld", regOperArr[registerOperation], ntransition);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rshmc8015_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the status reporting system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR registerOperation/This control selects the registers that will be configured.
/// HIPAR statusRegisterQuery/This control selects the registers that will be configured.
/// HIPAR registerValue/Returns content of selected register.
ViStatus _VI_FUNC rshmc8015_GetStatusRegister(ViSession instrSession,
                                              ViInt32   registerOperation,
                                              ViInt32   statusRegisterQuery,
                                              ViInt32*  registerValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, registerOperation, 1, 2),
            2, "Register Operation");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, statusRegisterQuery, 0, 4),
            2, "Status Register Query");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "STAT:%s:%s?", regOperArr[registerOperation], regTypeArr[statusRegisterQuery]);
    checkErr(rshmc8015_QueryViInt32(instrSession, cmd, registerValue));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Clears all bits in the Questionable Data enable register and the
/// HIFN  Standard Operation enable register.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rshmc8015_StatusPreset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rshmc8015_SetAttributeViString(instrSession, "", RSHMC8015_ATTR_STATUS_PRESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Get Error and Clear Error Functions
 * Purpose:  These functions enable the instrument driver user to
 *           get or clear the error information the driver associates with the
 *           RS session.
 *****************************************************************************/
ViStatus _VI_FUNC rshmc8015_GetError(ViSession instrSession,
                                     ViStatus* errorCode,
                                     ViInt32   bufferSize,
                                     ViChar    description[])
{  
    ViStatus error = VI_SUCCESS;

    // Lock Session - do not jump to the end, even with invalid instrSession the function continues further
    (void)(RsCore_LockSession(instrSession));

    // Test for nulls and acquire error data
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorCode), 2, "Null address for Error");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, description), 4, "Null address for Description");

    checkErr(RsCore_GetErrorCompleteDescription(instrSession, &rshmc8015_utility_error_message, errorCode, bufferSize, description)
    );

Error:
    // IGNORE: ERROR!!! Missing Lock
    return error;
}

ViStatus _VI_FUNC rshmc8015_ClearError(ViSession instrSession)
{
    return RsCore_ClearErrorInfo(instrSession);
}

/// HIFN This function writes a user-specified string to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.
/// HIPAR writeBuffer/Pass the string to be written to the instrument.
ViStatus _VI_FUNC rshmc8015_WriteInstrData(ViSession instrSession,
                                           ViString  writeBuffer)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    error = RsCore_Write(instrSession, writeBuffer);

    checkErr(rshmc8015_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the ViBoolean value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViBoolean value.
ViStatus _VI_FUNC rshmc8015_QueryViBoolean(ViSession instrSession,
                                           ViString  command,
                                           ViBoolean *value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, command, 1, -1),
        2, "Cmd (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value),
        3, "Value");

    checkErr(RsCore_QueryViBoolean(instrSession, command, value));

    checkErr(rshmc8015_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the ViInt32 value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViInt32 value.
ViStatus _VI_FUNC rshmc8015_QueryViInt32(ViSession instrSession,
                                         ViString  command,
                                         ViInt32   *value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, command, 1, -1),
        2, "Cmd (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViInt32(instrSession, command, value));

    checkErr(rshmc8015_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the ViReal64 value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViReal64 value.
ViStatus _VI_FUNC rshmc8015_QueryViReal64(ViSession instrSession,
                                          ViString  command,
                                          ViReal64  *value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, command, 1, -1),
        2, "Cmd (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViReal64(instrSession, command, value));

    checkErr(rshmc8015_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the ViString value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Value parameter.
/// HIPAR value/Returns the string value.
ViStatus _VI_FUNC rshmc8015_QueryViString(ViSession instrSession,
                                          ViString  command,
                                          ViInt32   bufferSize,
                                          ViChar    value[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, command, 1, -1),
        2, "Cmd (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, command, bufferSize, value, NULL));

    checkErr(rshmc8015_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The DATA? query responses screenshot data in binary format
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR destination/Defines destination file to which the data transfer from instrument to
/// HIPAR destination/control computer applies.
ViStatus _VI_FUNC rshmc8015_ImageSnapshot(ViSession instrSession,
                                          ViString  destination)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryBinaryDataBlockToFile(instrSession, "HCOP:DATA?", destination, RS_VAL_TRUNCATE));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the data format of the screenshot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR imageFormat/Selects the data format of the screenshot.
ViStatus _VI_FUNC rshmc8015_ConfigureImageFormat(ViSession instrSession,
                                                 ViInt32   imageFormat)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc8015_SetAttributeViInt32(instrSession, "", RSHMC8015_ATTR_HCOPY_IMAGE_FORMAT, imageFormat),
            2, "Image Format");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the width and height of the screenshot.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSHMC8015_ATTR_HCOPY_IMAGE_WIDTH
/// HIFN     RSHMC8015_ATTR_HCOPY_IMAGE_HEIGHT
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     HCOPy:SIZE:X?
/// HIFN     HCOPy:SIZE:Y?
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR imageWidth/Returns the width of the screenshot.
/// HIPAR imageHeight/Returns the height of the screenshot.
ViStatus _VI_FUNC rshmc8015_QueryImageSize(ViSession instrSession,
                                           ViInt32*  imageWidth,
                                           ViInt32*  imageHeight)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc8015_GetAttributeViInt32(instrSession, "", RSHMC8015_ATTR_HCOPY_IMAGE_WIDTH, imageWidth),
            2, "Image Width");

    viCheckParm(rshmc8015_GetAttributeViInt32(instrSession, "", RSHMC8015_ATTR_HCOPY_IMAGE_HEIGHT, imageHeight),
            3, "Image Height");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN If TRUE (default after init is FALSE), the driver calls SYST:ERR? automatically in CheckStatus() when Instrument Status Error is detected.
/// HIFN Use the function rshmc8015_GetError afterwards to get the detailed information about the instrument error(s).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoSystErrQuery/If TRUE (default after init is FALSE), the driver calls SYST:ERR? automatically in CheckStatus() when Instrument Status Error is detected.
ViStatus _VI_FUNC rshmc8015_ConfigureAutoSystemErrQuery(ViSession instrSession,
                                                        ViBoolean autoSystErrQuery)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->autoSystErrQuery = autoSystErrQuery;

Error:
    return error;
}

/// HIFN Session-related mutex switch.
/// HIFN If TRUE (default after init is FALSE), the driver sets a mutual-exclusion mechanism for every driver's hi-level function and attribute access.
/// HIFN Set this value to TRUE if you plan to use one session in more than one thread.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR multiThreadLocking/Session-related mutex switch
ViStatus _VI_FUNC rshmc8015_ConfigureMultiThreadLocking(ViSession instrSession,
                                                        ViBoolean multiThreadLocking)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->multiThreadLocking = multiThreadLocking;

Error:
    return error;
}

/// HIFN  This function returns the ID Query response string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IDQueryResponse/Returns the ID Query response string.
ViStatus _VI_FUNC rshmc8015_IDQueryResponse(ViSession instrSession,
                                            ViChar    IDQueryResponse[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rshmc8015_QueryViString(instrSession, "*IDN?", 1024, IDQueryResponse));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stops further command processing until all commands sent before
/// HIFN  *WAI have been executed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rshmc8015_ProcessAllPreviousCommands(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_Write(instrSession, "*WAI"));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the OPC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR OPC/Queries the OPC.
ViStatus _VI_FUNC rshmc8015_QueryOPC(ViSession instrSession,
                                     ViInt32*  OPC)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rshmc8015_QueryViInt32(instrSession, "*OPC?", OPC));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function clears status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rshmc8015_ClearStatus(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_ClearStatus(instrSession));
    checkErr(rshmc8015_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function specifies the minimum timeout value to use (in
/// HIFN milliseconds) when accessing the device associated with the
/// HIFN given session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.
/// HIPAR VISATimeout/This control sets the timeout value.
ViStatus _VI_FUNC rshmc8015_SetVISATimeout(ViSession instrSession,
                                           ViInt32   VISATimeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_SetVisaTimeout(instrSession, VISATimeout));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the minimum timeout value to use (in
/// HIFN milliseconds) when accessing the device associated with the
/// HIFN given session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rshmc8015_init or
/// HIPAR instrSession/rshmc8015_InitWithOptions function.
/// HIPAR VISATimeout/This control gets the timeout value.
ViStatus _VI_FUNC rshmc8015_GetVISATimeout(ViSession instrSession,
                                           ViInt32*  VISATimeout)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 temp;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetVisaTimeout(instrSession, &temp));
    *VISATimeout = (ViInt32)temp;

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Set OPC Timeout
 * Purpose:  Sets new OPC timeout
*****************************************************************************/
ViStatus _VI_FUNC rshmc8015_SetOPCTimeout(ViSession instrSession,
                                          ViInt32   timeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc8015_SetAttributeViInt32(instrSession, "", RS_ATTR_OPC_TIMEOUT, timeout),
            3, "Timeout");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Get OPC Timeout
 * Purpose:  Gets new OPC timeout
*****************************************************************************/
ViStatus _VI_FUNC rshmc8015_GetOPCTimeout(ViSession instrSession,
                                          ViInt32*  timeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rshmc8015_GetAttributeViInt32(instrSession, "", RS_ATTR_OPC_TIMEOUT, timeout),
            3, "Timeout");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function switches ON/OFF various error checkings performed inside of the driver.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR optionChecking/This control switches option checking On or Off.
/// HIPAR rangeChecking/This control enables or disables range checking.
/// HIPAR statusChecking/This control enables or disables instrument state checking
ViStatus _VI_FUNC rshmc8015_ConfigureErrorChecking(ViSession instrSession,
                                                   ViBoolean optionChecking,
                                                   ViBoolean rangeChecking,
                                                   ViBoolean statusChecking)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->optionChecking = optionChecking;
    checkErr(RsCore_SetAttributeViBoolean(instrSession, "", RS_ATTR_QUERY_INSTRUMENT_STATUS, 0, statusChecking));
    checkErr(RsCore_SetAttributeViBoolean(instrSession, "", RS_ATTR_RANGE_CHECK, 0, rangeChecking));

Error:
    return error;
}

/*****************************************************************************
 * Function: rshmc8015_close
 * Purpose:  This function closes the instrument.
 *
 *           Note:  This function must unlock the session before calling
 *           Rs_Dispose.
 *****************************************************************************/
ViStatus _VI_FUNC rshmc8015_close(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_ViClose(instrSession));

Error:
    (void)RsCore_Dispose(instrSession);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 *------------------- End Instrument Driver Source Code ---------------------*
 *****************************************************************************/
